function [V,I] = ComputeContValues(F,oldV,start_p,start_t);

global N Cdirect PROFILES;

% This function receives final values and active switches, and scrolls through
% each node to indicate how many subsequent moves ('I') player p will make in
% equilibrium, once at a particular node, and what would be his
% continuation value
% Notes:
% (a) If strat_p = 0 then each player assumes it is the other player who
% begins. Otherwise, it is known which player gets the first priority.
% (b) The continuation payoff are computed as of time 'start_t'.
% (c) The function assumes that switching costs are invariant to which moves 
% are being taken. It would be easy to extend the program to deal with richer
% cost technologies.

V = zeros(PROFILES,N);
I = zeros(PROFILES,N);

for k = 1:PROFILES,
    
   for p = 1:N, 
      
      node   = k;
      countI = 0;
      tempP  = start_p;
      if start_p==0, tempP=(3-p); end; % 3-p is the other player: p=1 -> 3-p=2, p=2 -> 3-p=1
         
      while sum(F(node,:))>0,
         if F(node,tempP)==0, tempP = 3-tempP; end;
         node = Destination(node,tempP,F(node,tempP));
         if tempP==p, countI = countI + 1; end;   
         tempP = 3-tempP;
      end;
                  
      V(k,p) = oldV(node,p) - countI*Cdirect(start_t);
      I(k,p) = countI;
         
   end;
   
end;
