function [p_star,t_star,sametflg] = FindNextStage(V,F);

global N K Cinv PROFILES T EPSILON;

% This function is the heart of the algorithm, and it is used to find
% (going backwards) the next critical point in time.
% The input to the function is the continuation values and the switches 
% already active. It returns the point in time ('t_star'), the player
% associated with it ('p_star'), and two flags that identify non-generic 
% cases.

pl = zeros(PROFILES,N);

DV = zeros(PROFILES,N,max(K)); % Value differences
DI = zeros(PROFILES,N,max(K)); % Differences in counts of subsequent moves
tt = zeros(PROFILES,N,max(K)); % latest time in which a move is profitable, for all pairs of moves
t  = zeros(PROFILES,N);        % latest time in which a move is profitable, for all originating profiles
x  = zeros(PROFILES,N);        % indicators of potential moves that should not be accounted for in searching the next stage

% This is an abuse of programming: we use the 'ComputeContValues' only to
% obtain the correct 'I' matrix. The 'V' matrix remains the same, by construction.
[V I] = ComputeContValues(F,V,0,0);

% This loops scrolls over all profiles and players, and potential switches.
% For each such a case, it computes the differences in V and I from being in 
% the destination profile vis-a-vis staying put, and keeps at 'tt' the latest 
% point in time in which such switches are profitable (values of 0 or -1
% imply that there is no such point in time, and value of 'T' implies that it
% is optimal immediately).
% It then scrolls over each originating profile, and finds the optimal switch among 
% all potential alternatives, and stores the latest time for it at 't'.
% When doing so, It turns on the 'x' variable if the optimal switch is a
% switch which is already active. If 'x' is turned on, the next part of the
% program will ignore this entry in its search for the next stage.

for k = 1:PROFILES,
   prof = decodei(k);      
   for p = 1:N,
      pl(k,p) = p;
      for a = 1:K(p),
          
         DV(k,p,a) = V(Destination(k,p,a),p) - V(k,p);
         DI(k,p,a) = I(Destination(k,p,a),p) - I(k,p);
         
         if DV(k,p,a) < 0, tt(k,p,a) = -1;
         elseif DV(k,p,a) == 0 & DI(k,p,a) > 0, tt(k,p,a) = -1;
         elseif DV(k,p,a) == 0 & DI(k,p,a) == -1, tt(k,p,a) = 0;
         elseif DI(k,p,a)<0, tt(k,p,a) = T;
         else, tt(k,p,a) = Cinv(DV(k,p,a)/(1+DI(k,p,a)));
         end;   
         
      end;
      tt = EPSILON*round(tt/EPSILON);
      tt(k,p,prof(p)) = -1;
      [t(k,p) des] = max(tt(k,p,:));
      if F(k,p)>0,
         if F(k,p)==des | DV(k,p,F(k,p)) >= DV(k,p,des), x(k,p) = 1; end;   
      end;   
   end;   
end;

% Turning on all 'x' matrix for a player with a full set of active
% switches. This simply expedites the termination of the program.

for p = 1:N,
   if sum(F(:,p)>0) == PROFILES*(K(p)-1)/K(p), x(:,p) = 1; end;   
end;

% Search for the latest critical point across profiles and players. Possible 
% switches with 'x' turned on are ignored.

p   = reshape(pl,PROFILES*N,1);
t   = reshape(t,PROFILES*N,1);
x   = reshape(x,PROFILES*N,1);

temp = [t p x];
temp = temp(x==0,:);
temp = sortrows(temp);

t   = temp(:,1);
p   = temp(:,2);

% Now t(lt) and p(lt) will contain the required t_star and p_star, and we
% just need to verify that there are no non-generic cases associated with it.

lt = length(t); 
count = 0;
while t(lt-count) == t(lt),
   count = count+1;
   if count==lt, break; end;   
end;

p_star   = p(lt+1-count:lt);
t_star   = t(lt);
sametflg = (min(p_star) < max(p_star)) & (t(lt) > 0);
p_star   = p_star(1);

   
  